//
//  RadarView.m
//  Ozz
//
//  Created by 高宇 on 16/11/18.
//  Copyright © 2016年 com.ozz. All rights reserved.
//

#import "RadarView.h"

@interface RadarView ()
@property (nonatomic,weak)CALayer *animationLayer;
@end
@implementation RadarView
-(instancetype)initWithFrame:(CGRect)frame
{
    if (self = [super initWithFrame:frame]) {
        //当重后台进入前台，防止假死状态
        [[NSNotificationCenter defaultCenter]addObserver:self selector:@selector(resume) name:UIApplicationDidBecomeActiveNotification object:nil];
        self.backgroundColor = [UIColor clearColor];
    }
    return self;
}

-(void)drawRect:(CGRect)rect
{
    [[UIColor clearColor]setFill];
    UIRectFill(rect);
    NSInteger pulsingCount = 2;
    double animationDuration = 2;
    
    CALayer * animationLayer = [[CALayer alloc]init];
    self.animationLayer = animationLayer;
    
    for (int i = 0; i < pulsingCount; i++) {
        
        
        //上面的两个半圆 半径为整个frame的四分之一
        CGFloat radius = MIN((self.frame.size.width-5*2)/4, (self.frame.size.height-5*2)/4);
        
        //左侧圆心 位于左侧边距＋半径宽度
        CGPoint leftCenter = CGPointMake(5+radius, 5+radius);
        //右侧圆心 位于左侧圆心的右侧 距离为两倍半径
        CGPoint rightCenter = CGPointMake(5+radius*3, 5+radius);
        
        //左侧半圆
        UIBezierPath *heartLine = [UIBezierPath bezierPathWithArcCenter:leftCenter radius:radius startAngle:M_PI endAngle:0 clockwise:YES];
        
        //右侧半圆
        [heartLine addArcWithCenter:rightCenter radius:radius startAngle:M_PI endAngle:0 clockwise:YES];
        
        //曲线连接到新的底部顶点 为了弧线的效果，控制点，坐标x为总宽度减spaceWidth，刚好可以相切，平滑过度 y可以根据需要进行调整，y越大，所画出来的线越接近内切圆弧
        [heartLine addQuadCurveToPoint:CGPointMake((self.frame.size.width/2), self.frame.size.height-5*2) controlPoint:CGPointMake(self.frame.size.width-5, self.frame.size.height*0.6-3)];
        
        //用曲线 底部的顶点连接到左侧半圆的左起点 为了弧线的效果，控制点，坐标x为spaceWidth，刚好可以相切，平滑过度。y可以根据需要进行调整，y越大，所画出来的线越接近内切圆弧（效果是越胖）
        [heartLine addQuadCurveToPoint:CGPointMake(5, 5+radius) controlPoint:CGPointMake(5, self.frame.size.height*0.6-3)];
        
        //根据坐标点连线
        //        [heartLine stroke];
        //clipToBounds 切掉多余的部分
        [heartLine addClip];
        
        CAShapeLayer *pulsingLayer = [[CAShapeLayer alloc] init];
        pulsingLayer.frame = CGRectMake(0, 0, rect.size.width, rect.size.height);
        pulsingLayer.path = heartLine.CGPath;
        pulsingLayer.fillColor = [UIColor redColor].CGColor;
        CAMediaTimingFunction * defaultCurve = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionLinear];
        
        CAAnimationGroup * animationGroup = [[CAAnimationGroup alloc]init];
        animationGroup.fillMode = kCAFillModeBoth;
        animationGroup.beginTime = CACurrentMediaTime() + (double)i * animationDuration/(double)pulsingCount;
        animationGroup.duration = animationDuration;
        animationGroup.repeatCount = HUGE_VAL;
        animationGroup.timingFunction = defaultCurve;
        
        CABasicAnimation * scaleAnimation = [CABasicAnimation animationWithKeyPath:@"transform.scale"];
        scaleAnimation.autoreverses = NO;
        scaleAnimation.fromValue = [NSNumber numberWithDouble:0.2];
        scaleAnimation.toValue = [NSNumber numberWithDouble:1.0];
        
        CAKeyframeAnimation * opacityAnimation = [CAKeyframeAnimation animationWithKeyPath:@"opacity"];
        opacityAnimation.values = @[[NSNumber numberWithDouble:1.0],[NSNumber numberWithDouble:0.5],[NSNumber numberWithDouble:0.3],[NSNumber numberWithDouble:0.0]];
        opacityAnimation.keyTimes = @[[NSNumber numberWithDouble:0.0],[NSNumber numberWithDouble:0.25],[NSNumber numberWithDouble:0.5],[NSNumber numberWithDouble:1.0]];
        animationGroup.animations = @[scaleAnimation,opacityAnimation];
        
        [pulsingLayer addAnimation:animationGroup forKey:@"pulsing"];
        [animationLayer addSublayer:pulsingLayer];
    }
    self.animationLayer.zPosition = -1;//重新加载时，使动画至底层
    [self.layer addSublayer:self.animationLayer];
    
}

-(void)resume
{
    if (self.animationLayer) {
        [self.animationLayer removeFromSuperlayer];
        [self setNeedsDisplay];
    }
}

/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
    // Drawing code
}
*/

@end
